/*
Copyright (c) 2004 Bart Upchurch

Permission is hereby granted, free of charge, to any person 
obtaining a copy of this software and associated documentation 
files (the "Software"), to deal in the Software without 
restriction, including without limitation the rights to use, 
copy, modify, merge, publish, distribute, sublicense, and/or 
sell copies of the Software, and to permit persons to whom 
the Software is furnished to do so, subject to the 
following conditions:

The above copyright notice and this permission notice shall 
be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR 
THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#include <Application.h>
#include <Window.h>
#include <View.h>
#include <Screen.h>
#include <String.h>
#include <Bitmap.h>
#include <unistd.h> //for sleep()

#include "MeasureDefs.h"
#include "MeasureApp.h"
#include "MeasureWindow.h"
#include "MeasureView.h"


/*void MeasureView::WindowActivated(bool active) {
	
		BScreen aScreen(B_MAIN_SCREEN_ID);	
		//int time = 0;
		
		if (active) {
			BWindow *aWindow; 
		
			aWindow = Window();
			aWindow->Minimize(true);
			//aWindow->Hide();
			//while (time < 10000000) {
			//	time++;
			//};
			sleep(1);
			if (aWindow->IsMinimized())
				aScreen.ReadBitmap(background,false);
			//aWindow->Show();
			aWindow->Minimize(false);
			Invalidate();
	    };
}
*/
void MeasureView::MouseMoved(BPoint point, uint32 transit, const BMessage *message) {
	
		mouseLocation = point;
		//only redraw for mouse movement if Toggle is flipped
		if (mouseToggle)
			Invalidate();
}

void MeasureView::KeyDown(const char *bytes, int32 numBytes) {

	
	BScreen aScreen(B_MAIN_SCREEN_ID);	
	aWindow = Window();
	
	if (numBytes == 1 ) {
		switch (bytes[0] ) {
			case B_SPACE:
				verticalGridToggle = 0;
				horizontalGridToggle = 0;
				break;
			case B_RIGHT_ARROW:
				verticalGridToggle += 1;
				break;
			case B_LEFT_ARROW:
				verticalGridToggle -= 1;
				break;
			case B_UP_ARROW:
				horizontalGridToggle += 1;
				break;
			case B_DOWN_ARROW:
				horizontalGridToggle -= 1;
				break;
			case 'f':
			case 'F':
				freezeToggle = !(freezeToggle);
				break;	
			case 'z':
			case 'Z':
				zoomToggle = !(zoomToggle);
				//hide Title Bar to go to Full Screen
				if (zoomToggle) {
					aWindow->SetLook(B_BORDERED_WINDOW_LOOK);
					aWindow->Zoom(); 
					}
				else {
					aWindow->SetLook(B_TITLED_WINDOW_LOOK);
					aWindow->MoveTo(200.0,200.0);
					aWindow->ResizeTo(200.0,100.0);
					};
				break;
			case 'm':
			case 'M':
				mouseToggle = !(mouseToggle);
				break;
			case 'l':
			case 'L':
				labelToggle = !(labelToggle);
				break;
			case 'h':
			case 'H':
				helpToggle = !(helpToggle);
				if (!(helpToggle)) {
					SetViewColor(B_TRANSPARENT_COLOR);
					SetLowColor(B_TRANSPARENT_COLOR);
				}
				else {
					SetViewColor(255,255,255); //white
					SetLowColor(255,255,255);
				};
				break;
			case 'r':
			case 'R':
				aWindow->Minimize(true);
				//aWindow->Hide();
				sleep(1);
				if (aWindow->IsMinimized())
					aScreen.ReadBitmap(background,false);
				//aWindow->Show();
				aWindow->Minimize(false);
			default:
				BView::KeyDown(bytes, numBytes);
				break;
		}
	
	Invalidate();
	}
}

void MeasureView::Draw(BRect updateRect) {

	BString horizontalLabel,
			verticalLabel,
			mouseLabel,
			updateRectLabel,
			srcRectLabel;
			
	BPoint	labelCoordinates;
	
	int vcount =0;
	int hcount =0;
	
	BPoint startLine, endLine;
	
	if (!(helpToggle)) {
		aWindow = Window();
	
		aWindow->Show();
		//freeze picture
	
		if (!(freezeToggle))
			*src = aWindow->Frame();	
		//redrae just the mouse portion of screen for performance
		//DO NOT NEED
		/*if (mouseToggle) {
			src->Set(src->left + updateRect.left + mouseLocation.x - 10,
				 src->top + updateRect.top + mouseLocation.y - 10,
				 src->left + updateRect.left + mouseLocation.x + 10,
				 src->top + updateRect.top + mouseLocation.y + 10);
			updateRect.Set(mouseLocation.x -10,
					   mouseLocation.y -10,
					   mouseLocation.x + 10,
					   mouseLocation.y +10);
		}*/
		//Makes the picture transparent even though outside of Screen
		if ((src->left > 0) && (src->top > 0)) 
			src->Set(src->left,src->top,src->left + updateRect.Width(), 
				src->top + updateRect.Height());
		else
			src->Set(src->right - updateRect.Width(), src->bottom - updateRect.Height(),
				src->right, src->bottom);
			
		DrawBitmap(background,*src,updateRect);
	
		SetPenSize(1.0);
	
		//Create Screen and Frame Coordinates of Window
		if (labelToggle) {
			updateRectLabel << "(" << updateRect.left << ", " << updateRect.top 
				<< ", " << updateRect.right << "," << updateRect.bottom << ")";
			DrawString(updateRectLabel.String(), updateRect.LeftBottom());
	
			srcRectLabel << "(" << src->left << ", " << src->top 
				<< ", " << src->right << "," << src->bottom << ")";
			labelCoordinates.Set(0,10);
			DrawString(srcRectLabel.String(), labelCoordinates);
		};
	
		//Create Grid
		verticalGridToggle = verticalGridToggle % 8;
		horizontalGridToggle = horizontalGridToggle % 8;
	
	
		if (horizontalGridToggle != 0) {
			switch (horizontalGridToggle) {
				case 1:
					hcount = 10;
					break;
				case 2:
					hcount = 20;
					break;
				case 3:
					hcount = 25;
					break;
				case 4:
					hcount = 50;
					break;
				case 5:
					hcount = 75;
					break;
				case 6:
					hcount = 100;
					break;
				case 7:
					hcount = 150;
					break;
			}
		
			horizontalLabel << hcount;
			labelCoordinates.Set(0.0,20.0);
			DrawString(horizontalLabel.String(), labelCoordinates);
		
			for (int count=hcount; count < updateRect.IntegerHeight(); count+=hcount)
				{
				startLine.Set(0.0,count);
				endLine.Set(updateRect.Width(),count);
				StrokeLine(startLine,endLine);
			};	
		}
	
		if (verticalGridToggle != 0) {
			switch (verticalGridToggle) {
				case 1:
					vcount = 10;
					break;
				case 2:
					vcount = 20;
					break;
				case 3:
					vcount = 25;
					break;
				case 4:
					vcount = 50;
					break;
				case 5:
					vcount = 75;
					break;
				case 6:
					vcount = 100;
					break;
				case 7:
					vcount = 150;
					break;
			}
		
			verticalLabel << vcount;
			labelCoordinates.Set(10.0,10.0);
			DrawString(verticalLabel.String(), labelCoordinates);
		
			for (int count=vcount; count < updateRect.IntegerWidth(); count+=vcount)
				{
				startLine.Set(count,0.0);
				endLine.Set(count,updateRect.Height());
				StrokeLine(startLine,endLine);
			};		
	
		};
	
		if (mouseToggle) {
			mouseLabel << "(" << int(mouseLocation.x) << ", " << int(mouseLocation.y) << ")";
		
			//Move label to be seen when at the top of a View
			if ((mouseLocation.x > 60) && (mouseLocation.y > 20)) {
				mouseLocation.Set(mouseLocation.x - 60,mouseLocation.y - 10);
				DrawString(mouseLabel.String(), mouseLocation);
			}
			else {
				mouseLocation.Set(mouseLocation.x + 20,mouseLocation.y + 20);
				DrawString(mouseLabel.String(), mouseLocation);
			};
		};
	} 
	//display help menu
	else {
		labelCoordinates.Set((updateRect.Width()/2 - 25),(updateRect.Height()/10) * 1);
		DrawString("Measure", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 2);
		DrawString("Use Arrow Keys to toggle grid", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 3);
		DrawString("Use Space to Clear grid", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 4);
		DrawString("Use 'm' or 'M' to get mouse Coordinates", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 5);
		DrawString("Use 'f' of 'F' to freeze the image", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 6);
		DrawString("Use 'l' or 'L' to get the Window and Frame Coordinates", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 7);
		DrawString("Use 'z' or 'Z' for Zoom to measure Screen Coordinates", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 8);
		DrawString("Use 'r' or 'R' to Refresh background", labelCoordinates);
		labelCoordinates.Set(0.0,(updateRect.Height()/10) * 9);
		DrawString("Use 'h' or 'H' to toggle help", labelCoordinates);
	};
}

void MeasureView::AttachedToWindow(void) {
	
	BScreen aScreen(B_MAIN_SCREEN_ID);
	
	//aWindow = Window();
	
	//aWindow->Minimize(true);
	//Grap a ScreenShot to fake transprancy

	aScreen.ReadBitmap(background,false);		
	//aWindow->LockLooper();
	//aWindow->Minimize(false);
	//aWindow->UnlockLooper();
	
	//Smooth Drawing
	
	BView::AttachedToWindow();
	

}

MeasureView::MeasureView(BRect frame, const char *name)
			: BView(frame, name, B_FOLLOW_ALL  | B_FRAME_EVENTS, B_WILL_DRAW) {
	
	BScreen aScreen(B_MAIN_SCREEN_ID);
	
	verticalGridToggle = 0;
	horizontalGridToggle = 0;
	freezeToggle = false;
	zoomToggle = false;
	mouseToggle = false;
	labelToggle = false;
	helpToggle = true;
	
	
	src = new BRect(frame);
	background = new BBitmap(aScreen.Frame(),aScreen.ColorSpace());
	
}

MeasureView::~MeasureView() {
	delete src;
	delete background;
}